@@ -56,6 +56,7 @@ using std::vector;
 #define DEFAULT_CERT_WARNING 60
 #define DEFAULT_ADIF_EDIT false
 #define DEFAULT_DISP_DUPES true
+#define DEFAULT_IGNORE_SECONDS false
 #define DEFAULT_LOG_TAB false
 #define DEFAULT_CERTPWD false
 //online
@@ -92,6 +93,7 @@ enum {		// Window IDs
 	ID_PREF_FILE_LOG_TAB,
 	ID_PREF_FILE_CERTPWD,
 	ID_PREF_FILE_LOGVFY,
+	ID_PREF_IGNORE_SECONDS,
 	ID_PREF_MODE_MAP,
 	ID_PREF_MODE_ADIF,
 	ID_PREF_MODE_DELETE,
@@ -152,7 +154,7 @@ class LogPrefs : public PrefsPanel {
 	void ShowHide();
  private:
 	wxTextCtrl *cabrillo, *adif;
-	wxCheckBox *badcalls, *daterange, *dispdupes;
+	wxCheckBox *badcalls, *daterange, *dispdupes, *ignoresecs;
 	wxRadioBox *handleQTH;
 };
 
